library IEEE;
use IEEE.std_logic_1164.all;
use ieee.std_logic_textio.all;
use IEEE.numeric_std.all;

library std;
use std.textio.all;

entity Lab5TestBench is
  
end Lab5TestBench;

architecture Lab5_arch of Lab5TestBench is

  component calculator is
    port(
      binary : in std_logic_vector(3 downto 0);	-- binary rep of input
      isnum : in std_logic;			-- whether the input is a number
      rst : in std_logic;			-- reset signal
      kb_prs : in std_logic;			-- ready signal
      valid : in std_logic;			-- valid signal
      result : out std_logic_vector(7 downto 0);-- binary rep of output
      mode: out std_logic_vector(3 downto 0)	-- current state
      );
  end component calculator;
  
  component kb_to_binary is
    port (
      sc: in STD_LOGIC_VECTOR (7 downto 0);	-- input scancode
      bin: out STD_LOGIC_VECTOR (3 downto 0);	-- binary representation
      isnum: out STD_LOGIC;			-- whether input is a number
      kb_prs: in STD_LOGIC;			-- keyboard press signal
      clk: in STD_LOGIC;			-- clock
      valid: out STD_LOGIC			-- whether input is valid
      );
  end component kb_to_binary;
  
  --------------------------------------------------------
  -- structural signals required to connect the components
  --------------------------------------------------------
  signal binary_sig : std_logic_vector(3 downto 0);
  signal isnum_sig : std_logic;
  signal kb_prs_sig : std_logic := '0';
  signal valid_sig: std_logic;
  signal reset_sig: std_logic := '1';
  signal scancode_sig: std_logic_vector(7 downto 0);
  signal result_sig: std_logic_vector(7 downto 0);
  signal mode_sig: std_logic_vector(3 downto 0);
  signal clk: std_logic := '0';
  
  --------------------------------------------------------
  -- simulator variables, constants, etc.
  --------------------------------------------------------
  constant clk_period : time := 5 ns;
  signal done_sim : boolean := false;
  signal done : boolean := false;
  shared variable scancode_buf: std_logic_vector(7 downto 0);
  shared variable failed_v: integer := 0;
  
begin
  dut : calculator
    port map (
      binary => binary_sig,
      isnum => isnum_sig,
      rst => reset_sig,
      kb_prs => kb_prs_sig,
      valid => valid_sig,
      result => result_sig,
      mode => mode_sig
      );
  kb2bin: kb_to_binary
    port map (
      sc => scancode_sig,
      bin => binary_sig,
      isnum => isnum_sig,
      kb_prs => kb_prs_sig,
      clk => clk,
      valid => valid_sig
      );

-- the clock upon which we synchronize all of our processes
  clk <= not clk after clk_period when not done else '0';
  kb_prs_sig <= not kb_prs_sig after 3*clk_period when not done else '0';


  -----------------------------------------------------------------------------
  -- TestProcess: Feeds input to the dut
  -----------------------------------------------------------------------------
  TestProcess: process(kb_prs_sig)
    variable input_line_v: line;
    variable input_sc_v: std_logic_vector(7 downto 0);
    file input: text is "Input.txt";
  begin
    if now > 2*clk_period then
      reset_sig <= '0';
    end if;
    
    if kb_prs_sig'event and kb_prs_sig = '1' then
      if not endfile(input) then
        readline(input, input_line_v);
        hread(input_line_v, input_sc_v);
        scancode_sig <= input_sc_v;
      else
        done_sim <= true;
      end if;
    end if;
  end process;

  -----------------------------------------------------------------------------
  -- ScancodeBufProcess:  Checks scancode when enter is pressed
  -----------------------------------------------------------------------------
  
  ScancodeBufProcess: process(clk)
    variable lout: line;
    variable output_line_v: line;
    variable expected_output_v: std_logic_vector(7 downto 0);
    file output1: text is "output1.txt";
  begin
    if clk'event and clk='0' then
      if scancode_buf = x"5a" 
        and scancode_sig /= scancode_buf then 
        -- enter has defined output, therefore we
        -- only check on enter
        if not endfile(output1) then
          
          readline(output1,output_line_v);
          hread(output_line_v, expected_output_v);
          
          if expected_output_v /= result_sig then
            
--    					write(lout, string'("Expected: "));
--    					write(lout, expected_output_v);
--    					write(lout, string'(", Actual: "));
--    					write(lout, result_sig);
--    					writeline(output, lout);

            failed_v := failed_v + 1;
            report "Output mismatch" severity error;
            
          end if;
        end if;
      end if;
      scancode_buf := scancode_sig;
    end if;
  end process;
  
  
  ----------------------------------------------------------
  -- ReportProcess: reports the results of the simulation
  ----------------------------------------------------------
  ReportProcess: process(done_sim)
    variable lout : line;
  begin
    if done_sim then
      write(lout, string'("Note: Failed "));
      write(lout, failed_v);
      write(lout, string'(" trials."));
      writeline(output, lout);
      done <= true;
    end if;
  end process;
  
end Lab5_arch;















